<?php
/**
 * @copyright: Copyright © 2017 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Console\Command;

use Magento\Framework\Console\Cli;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Command\Command;
use Magento\Sales\Model\OrderRepository;
use Firebear\ImportExport\Model\Migration\ProgressHelperFactory;

/**
 * Command prints list of available currencies
 */
class OrderSend extends Command
{
    const ARGUMENT_NAME = 'order';

    const ARGUMENT_ALL = 'all';

    protected $eventManager;

    protected $orderRepository;

    protected $searchCriteriaBuilder;

    protected $filterGroupBuilder;

    protected $filterBuilder;

    protected $publisher;

    protected $progressHelperFactory;

    protected $topicName;

    public function __construct(
        \Magento\Framework\Event\ManagerInterface $eventManager,
        OrderRepository $orderRepository,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\Search\FilterGroupBuilder $filterGroupBuilder,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        \Magento\Framework\MessageQueue\PublisherInterface $publisher,
        ProgressHelperFactory $progressHelperFactory,
        $topicName = 'sap.order.update'
    ) {
        $this->eventManager = $eventManager;
        $this->orderRepository = $orderRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterGroupBuilder = $filterGroupBuilder;
        $this->filterBuilder = $filterBuilder;
        $this->publisher = $publisher;
        $this->progressHelperFactory = $progressHelperFactory;
        $this->topicName = $topicName;
        parent::__construct();
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName('sap:order:send')
            ->setDescription('Send Magento Order to SBO');

        $this->addArgument(
            self::ARGUMENT_NAME,
            InputArgument::IS_ARRAY | InputArgument::OPTIONAL,
            'Space-separated list of order ids'
        );
        $this->addOption(
            self::ARGUMENT_ALL,
            null,
            InputOption::VALUE_NONE,
            '--all for all orders'
        );

        parent::configure();
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $ids = $input->getArgument(self::ARGUMENT_NAME);
        $filterGroups = [];
        $statusFilter = $this->filterBuilder
            ->setField('status')
            ->setValue('pending')
            ->create();
        $filterGroups[] = $this->filterGroupBuilder
            ->setFilters([$statusFilter])
            ->create();

//        $sboFilter = $this->filterBuilder
//            ->setField(\Firebear\PlatformSap\Model\Export\Adapter\Order::DOCNUM_FIELD)
//            ->setConditionType('null')
//            ->create();
//        $filterGroups[] = $this->filterGroupBuilder
//            ->setFilters([$sboFilter])
//            ->create();

        if ($ids) {
            $idFilter = $this->filterBuilder
                ->setField('entity_id')
                ->setValue($ids)
                ->setConditionType('in')
                ->create();
            $filterGroups[] = $this->filterGroupBuilder
                ->setFilters([$idFilter])
                ->create();
        }

        $criteria = $this->searchCriteriaBuilder
            ->setFilterGroups($filterGroups)
            ->create();

        $orders = $this->orderRepository->getList($criteria);

        $progressHelper = $this->progressHelperFactory->create([
            'output' => $output,
            'name' => 'Send orders to queue',
            'max' => $orders->getSize(),
        ]);

        foreach ($orders as $order) {
            $this->publisher->publish($this->topicName, $order->getId());
            $progressHelper->advance(1);
        }

        $progressHelper->finish();

        return Cli::RETURN_SUCCESS;
    }
}
