<?php
/**
 * @copyright: Copyright © 2020 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Controller\Adminhtml\Customer;

use Psr\Log\LoggerInterface;
use Magento\Framework\Exception\LocalizedException;
use Firebear\PlatformSap\Model\Customer\UpdatePublisher;
use Magento\Customer\Model\ResourceModel\CustomerRepository as CustomerRepository;
use Magento\Backend\App\Action\Context;

class Sync extends \Magento\Backend\App\Action
{
    const ADMIN_RESOURCE = 'Firebear_PlatformSap::index';

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var UpdatePublisher
     */
    private $customerUpdatePublisher;

    /**
     * @var CustomerRepository
     */
    private $customerRepository;

    /**
     * Sync constructor.
     * @param Context $context
     * @param LoggerInterface $logger
     * @param UpdatePublisher $customerUpdatePublisher
     * @param CustomerRepository $customerRepository
     */
    public function __construct(
        Context $context,
        LoggerInterface $logger,
        UpdatePublisher $customerUpdatePublisher,
        CustomerRepository $customerRepository
    ) {
        $this->customerUpdatePublisher = $customerUpdatePublisher;
        $this->customerRepository = $customerRepository;
        $this->logger = $logger;
        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\Result\Redirect|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $resultRedirect = $this->resultRedirectFactory->create();
        try {
            $customerId = $this->getRequest()->getParam('customer_id');
            $customer = $this->customerRepository->getById($customerId);
            $this->customerUpdatePublisher->execute($customer);
            $message = __('The customer has been added to the queue for sync with SAP');
            $this->messageManager->addSuccessMessage($message);
            $resultRedirect->setPath('customer/index/edit', ['id' => $customerId]);
        } catch (LocalizedException $e) {
            $this->logger->critical($e);
            $this->messageManager->addErrorMessage($e->getMessage());
            return $resultRedirect->setPath('customer/index/index');
        } catch (\Exception $e) {
            $this->logger->critical($e);
            $this->messageManager->addException($e, __('Error while processing order.'));
            return $resultRedirect->setPath('customer/index/index');
        }
        return $resultRedirect;
    }
}
