<?php
/**
 * @copyright: Copyright © 2021 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */
namespace Firebear\PlatformSap\Cron;

use Firebear\PlatformSap\Model\Customer\UpdatePublisher as CustomerUpdatePublisher;
use Magento\Customer\Model\ResourceModel\CustomerRepository;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Firebear\PlatformSap\Model\LogRepository;
use Firebear\PlatformSap\Model\Order\UpdatePublisher as OrderUpdatePublisher;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 * Class LogRestart
 * @package Firebear\PlatformSap\Cron
 */
class LogRestart
{
    /**
     * @var FilterBuilder
     */
    private $filterBuilder;

    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    /**
     * @var LogRepository
     */
    private $logRepositiry;

    /**
     * @var CustomerRepository
     */
    private $customerRepository;

    /**
     * @var CustomerUpdatePublisher
     */
    private $customerUpdatePublisher;

    /**
     * @var OrderUpdatePublisher
     */
    private $orderUpdatePublisher;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * LogCleanup constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param FilterBuilder $filterBuilder
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param LogRepository $logRepository
     * @param CustomerRepository $customerRepository
     * @param CustomerUpdatePublisher $customerUpdatePublisher
     * @param OrderUpdatePublisher $orderUpdatePublisher
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        FilterBuilder $filterBuilder,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        LogRepository $logRepository,
        CustomerRepository $customerRepository,
        CustomerUpdatePublisher $customerUpdatePublisher,
        OrderUpdatePublisher $orderUpdatePublisher
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->filterBuilder = $filterBuilder;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->logRepositiry = $logRepository;
        $this->customerRepository = $customerRepository;
        $this->customerUpdatePublisher = $customerUpdatePublisher;
        $this->orderUpdatePublisher = $orderUpdatePublisher;
    }

    /**
     * Restart all sap integration logs which have an error status
     * @return void
     */
    public function execute()
    {
        $restartedLogs = [];
        $filter =  $this->filterBuilder->setField('status')
            ->setValue(1)
            ->setConditionType('eq')
            ->create();
        $logs = (array)($this->logRepositiry->getList(
            $this->searchCriteriaBuilder->addFilters([$filter])->create()
        )->getItems());

        if (!empty($logs)) {
            foreach ($logs as $log) {
                $entityId = $log->getEntityId();
                if (!isset($restartedLogs['customer'][$entityId])
                    && $log->getEntity() == 'customer'
                    && $this->scopeConfig->getValue('firebear_importexport/sap/export_customers')
                ) {
                    try {
                        $customer = $this->customerRepository->getById($entityId);
                        $this->customerUpdatePublisher->execute($customer);
                        $restartedLogs['customer'][$entityId] = true;
                    } catch (NoSuchEntityException $e) {
                        $restartedLogs['customer'][$entityId] = true;
                    }
                } elseif (!isset($restartedLogs['order'][$entityId])
                    && $log->getEntity() == 'order'
                    && $this->scopeConfig->getValue('firebear_importexport/sap/sap_export_orders')) {
                    $this->orderUpdatePublisher->execute($entityId);
                    $restartedLogs['order'][$entityId] = true;
                }
            }
        }
    }
}
