<?php
/**
 * @copyright: Copyright © 2020 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Model\Export\Adapter\Gateway;

use Firebear\ImportExport\Api\Export\History\CompressInterface;
use Firebear\ImportExport\Logger\Logger;
use Firebear\ImportExport\Model\Filesystem\File\ReadFactory;
use Firebear\ImportExport\Model\Source\Factory as SourceFactory;
use Firebear\ImportExport\Traits\General as GeneralTrait;
use Firebear\PlatformSap\Model\LogRepository;
use Magento\Customer\Model\ResourceModel\AddressRepository;
use Magento\Customer\Model\ResourceModel\CustomerRepository;
use Magento\Framework\App\CacheInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Directory\WriteFactory as DirectoryWriteFactory;
use Magento\Framework\Filesystem\File\WriteFactory as FileWriteFactory;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Stdlib\DateTime\Timezone;
use Magento\Sales\Model\OrderRepository;
use Symfony\Component\Console\Output\ConsoleOutput;
use Magento\Directory\Model\CountryFactory;
use Magento\Sales\Model\Order\Status\HistoryFactory;

/**
 * Sap customer gateway
 */
class SalesAbstractGateway extends AbstractGateway
{
    /**
     * @var CountryFactory
     */
    protected $countryFactory;

    /**
     * @var HistoryFactory
     */
    protected $statusHistoryFactory;

    /**
     * SalesAbstractGateway constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param Filesystem $filesystem
     * @param ReadFactory $readFactory
     * @param DirectoryWriteFactory $writeFactory
     * @param FileWriteFactory $fileWrite
     * @param Timezone $timezone
     * @param SourceFactory $factory
     * @param CacheInterface $cache
     * @param LogRepository $logRepository
     * @param CompressInterface $compressCommand
     * @param CountryFactory $countryFactory
     * @throws \Magento\Framework\Exception\FileSystemException
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Filesystem $filesystem,
        ReadFactory $readFactory,
        DirectoryWriteFactory $writeFactory,
        FileWriteFactory $fileWrite,
        Timezone $timezone,
        SourceFactory $factory,
        CacheInterface $cache,
        LogRepository  $logRepository,
        CompressInterface $compressCommand,
        CountryFactory $countryFactory
    ) {
        parent::__construct(
            $scopeConfig,
            $filesystem,
            $readFactory,
            $writeFactory,
            $fileWrite,
            $timezone,
            $factory,
            $cache,
            $logRepository,
            $compressCommand
        );
        $this->countryFactory = $countryFactory;
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return array
     */
    protected function getOrderAddresses(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        $billStreet = $order->getBillingAddress()->getStreet();
        $shipStreet = $order->getShippingAddress()->getStreet();
        $billStreet0 = isset($billStreet[0]) ? $billStreet[0] : '';
        $billStreet1 = isset($billStreet[1]) ? $billStreet[1] : '';
        $shipStreet0 = isset($shipStreet[0]) ? $shipStreet[0] : '';
        $shipStreet1 = isset($shipStreet[1]) ? $shipStreet[1] : '';
        $addresses = [
            'BillToStreet' => $billStreet0,
            'BillToStreetNo' => $billStreet1,
            'BillToCity' => $order->getBillingAddress()->getCity(),
            'BillToZipCode' => $order->getBillingAddress()->getPostcode(),
            'BillToCountry' => $order->getBillingAddress()->getCountryId(),
            'ShipToStreet' => $shipStreet0,
            'ShipToStreetNo' => $shipStreet1,
            'ShipToCity' => $order->getShippingAddress()->getCity(),
            'ShipToZipCode' => $order->getShippingAddress()->getPostcode(),
//            @TODO map SAP State with Magento region_id
//            'BillToState' => substr($order->getBillingAddress()->getRegionCode(), 0, 2),
//            'ShipToState' => substr($order->getShippingAddress()->getRegionCode(), 0, 2),
            'ShipToCountry' => $order->getShippingAddress()->getCountryId()
        ];
        $billingCountry = $this->countryFactory->create()->loadByCode($order->getBillingAddress()->getCountryId());
        $addresses['billing_address_text'] = implode(' ', [
            $billStreet0,
            $billStreet1,
            $order->getBillingAddress()->getPostcode(),
            $order->getBillingAddress()->getCity(),
            $billingCountry->getName()
        ]);
        $shippingCountry = $this->countryFactory->create()->loadByCode($order->getShippingAddress()->getCountryId());
        $addresses['shipping_address_text'] = implode(' ', [
            $shipStreet0,
            $shipStreet1,
            $order->getShippingAddress()->getPostcode(),
            $order->getShippingAddress()->getCity(),
            $shippingCountry->getName()
        ]);

        return $addresses;
    }
}
