<?php
/**
 * @copyright: Copyright © 2021 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Model\Source;

use Magento\Framework\Data\OptionSourceInterface;
use Magento\Framework\Escaper;
use Magento\Store\Model\System\Store as SystemStore;

/**
 * Options tree for "Website" field
 */
class Website implements OptionSourceInterface
{
    /**
     * All Store Views value
     */
    const ALL_WEBSITES = 'All';

    /**
     * @var array
     */
    private $options;

    /**
     * System store
     *
     * @var SystemStore
     */
    protected $systemStore;

    /**
     * Escaper
     *
     * @var Escaper
     */
    protected $escaper;

    /**
     * Website constructor.
     * @param SystemStore $systemStore
     * @param Escaper $escaper
     */
    public function __construct(
        SystemStore $systemStore,
        Escaper $escaper
    ) {
        $this->systemStore = $systemStore;
        $this->escaper = $escaper;
    }

    /**
     * @return array
     */
    public function toOptionArray()
    {
        return $this->getAllOptions();
    }

    /**
     * @return array
     */
    public function getAllOptions()
    {
        if (!$this->options) {
            $websiteCollection = $this->systemStore->getWebsiteCollection();

            foreach ($websiteCollection as $website) {
                $name = $this->escaper->escapeHtml($website->getName());
                $currentOptions[$name]['label'] = $name;
                $currentOptions[$name]['value'] = $website->getCode();
            }

            $this->options = array_values($currentOptions);
        }
        return $this->options;
    }
}
