<?php
/**
 * @copyright: Copyright © 2020 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Plugin;

use Firebear\PlatformSap\Model\Customer\UpdatePublisher;
use Magento\Customer\Model\ResourceModel\CustomerRepository as CustomerRepository;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Customer\Controller\Address\FormPost;
use Magento\Customer\Model\Session;

/**
 * Class CustomerUpdatePlugin
 * @package Firebear\PlatformSap\Plugin
 */
class CustomerUpdatePlugin
{
    /**
     * @var UpdatePublisher
     */
    private $customerUpdatePublisher;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * @var Session
     */
    private $customerSession;

    /**
     * @var CustomerRepository
     */
    private $customerRepository;

    /**
     * CustomerUpdatePlugin constructor.
     * @param UpdatePublisher $customerUpdatePublisher
     * @param ScopeConfigInterface $scopeConfig
     * @param Session $customerSession
     * @param CustomerRepository $customerRepository
     */
    public function __construct(
        UpdatePublisher $customerUpdatePublisher,
        ScopeConfigInterface $scopeConfig,
        Session $customerSession,
        CustomerRepository $customerRepository
    ) {
        $this->customerUpdatePublisher = $customerUpdatePublisher;
        $this->scopeConfig = $scopeConfig;
        $this->customerSession = $customerSession;
        $this->customerRepository = $customerRepository;
    }

    /**
     * @param FormPost $subject
     * @param \Magento\Framework\Controller\Result\Redirect $result
     * @return \Magento\Framework\Controller\Result\Redirect
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function afterExecute(
        FormPost $subject,
        \Magento\Framework\Controller\Result\Redirect $result
    ) {
        if ($this->scopeConfig->getValue('firebear_importexport/sap/export_customers')) {
            $customerId = $this->customerSession->getCustomerId();
            $customer = $this->customerRepository->getById($customerId);
            $this->customerUpdatePublisher->execute($customer);
        }
        return $result;
    }
}
